# A generic cache.
# Stores key-value pairs.
# This is pretty much a more comfortable hash object.
class slender_data.Cache

  constructor: ->

    # Container for the cached objects.
    @cache = {}


  # Adds the given entry to the cache.
  # Overwrites existing entries.
  add: (key, value) ->
    @cache[key] = value


  # Returns the entry with the given key from the cache, or NULL if no entry exists.
  get: (key) ->
    @cache[key]


  # Looks up several entries at once.
  # Returns a hash of found entries, and a list of missing entries.
  get_many: (keys) ->
    found = []
    missing = []
    for key in keys
      if (entry = @cache[key])
        found.push entry
      else
        missing.push key
    found: found, missing: missing


  # Returns the number of cached objects.
  length: ->
    slender_data.object_length @cache


  # Removes the entry with the given key.
  remove: (key) =>
    delete @cache[key]


  # Removes all entries with the given keys.
  remove_many: (keys) ->
    @remove(key) for key in keys
