# Base class for all classes that are configured by a SlenderData configuration object.
class slender_data.Parameterizable

  constructor: (params) ->

    # The name of a data entry,
    # i.e. "user".
    @entry_name = params.entry_name ? throw new Error('You must provide an entry name')

    # Name of multiple data entries,
    # i.e. "users".
    @entries_name = params.entries_name ? "#{@entry_name}s"

    # Name of the id field in entries.
    @id_name = params.id_name ? 'id'

    # Name of the ids field in entries.
    @ids_name = params.ids_name ? "#{@id_name}s"

    # The method to use to check whether a given entry is full or partial.
    @is_full_entry = params.full_entry_checker

    # The class that is used to represent multi-queries.
    @multi_query_class = params.multi_query_class ? slender_data.MultiQuery

    # The given parameters.
    @params = params

    # Whether partial entry supported is enabled.
    @partial_entries = params.partial_entries ? no

    # The class that is used to represent single queries.
    @single_query_class = params.single_query_class ? slender_data.SingleQuery

    # Name of the version attribute,
    # i.e. "timestamp".
    @version_name = params.version_name ? 'version'

    # Whether versioning should be supported or not.
    @versioning = params.versioning ? no


  # Makes sure the given query is indeed a SingleQuery.
  _querify_single: (query) ->
    slender_data.querify query, @single_query_class, @params

  # Makes sure the given query is indeed a MultiQuery.
  _querify_multi: (multi_query) ->
    slender_data.querify multi_query, @multi_query_class, @params

