#= require slender_data/queries/base_query
#= require slender_data/queries/single_query


# Represents a query for multiple entries at once.
class slender_data.MultiQuery extends slender_data.BaseQuery

  constructor: (query, params) ->
    super query, params

    # Cache for the currently known entry ids of this query.
    # If undefined, this value will be calculated later on demand.
    #
    # Don't use this field directly. Use ids() to access it.
    @_ids_value = undefined

    # Cache for the currently known entries of this query.
    # If undefined, this value will be calculated later on demand.
    #
    # Don't use this field directly. Use entries() to access it.
    @_entries_value = undefined

    @_store_multi_query query

    # Allows to access the entries through their domain name,
    # i.e. @users()
    @[@entries_name] = @entries

    # Allows to access the entry ids through their domain name,
    # i.e. usernames()
    @[@ids_name] = @ids


  # Returns whether this multi-query describes no entries.
  empty: ->
    @ids().length is 0


  # Returns the corresponding entries for this query.
  entries: ->
    @_entries_value or throw new Error("Cannot provide #{@entries_name} for a #{@ids_name} query")


  # Returns the ids for this query.
  ids: ->
    @_ids_value or= (entry[@id_name] for entry in @entries())


  # Creates a new MultiQuery with the same configuration as this one,
  # but the given selector value.
  replicate: (new_selector_value) ->
    replica_query = _.clone @given_query
    replica_query[@selector_name()] = new_selector_value
    result = new @multi_query_class replica_query, @params
    result.full = @full
    result


  # Converts the given multi-key name into the corresponding single key name.
  single_selector_name: ->
    unless @_single_selector_name_result
      if @selector_name() is @entries_name
        @_single_selector_name_result = @entry_name
      else
        @_single_selector_name_result = @selector_name().substring 0, @selector_name().length-1
    @_single_selector_name_result


  # Stores the given data.
  _store_multi_query: (multi_query) ->
    switch @selector_name()
      when 'entries'
        @_entries_value = multi_query.entries
      when @entries_name
        @_entries_value = multi_query[@entries_name]
      when 'ids'
        @_ids_value = multi_query.ids
      when @ids_name
        @_ids_value = multi_query[@ids_name]
      else
        throw new Error("Unknown multi-query key name: #{@selector_name()}")


  # Converts the given multi-query into an array of single queries.
  transpose: ->
    result = []
    for selector in @selector_value()
      single_query = {}
      single_query[@single_selector_name()] = selector
      single_query.full = yes if @full
      result.push new @single_query_class(single_query, @params)
    result

