#= require slender_data/queries/base_query


# Represents a query for a single entry.
class slender_data.SingleQuery extends slender_data.BaseQuery

  constructor: (query, params) ->
    super query, params

    # Cache for the currently known entry id of this query.
    # If undefined, this value will be calculated later on demand.
    #
    # Don't use this field directly. Use id() to access it.
    @_id_value = undefined

    # Cache for the currently known entry of this query.
    # If undefined, this value will be calculated later on demand.
    #
    # Don't use this field directly. Use entry() to access it.
    @_entry_value = undefined

    # Cache for the currently known entry version for this query.
    # If undefined, this value will be calculated later on demand.
    #
    # Don't use this field directly. Use version() to access it.
    @_version_value = undefined

    @_store_query query

    # Allows to access the entry through its domain name,
    # i.e. @user()
    @[@entry_name] = @entry

    # Allows to access the entry id through its domain name,
    # i.e. username()
    @[@id_name] = @id

    # Allows to access the version through its domain name,
    # i.e. timestamp()
    @[@version_name] = @version


  # Returns the data entry of this query.
  # This method will be accessed through an alias named after the model name.
  entry: ->
    if @_entry_value
      @_entry_value
    else
      throw new Error "Cannot provide the entry because it was not given to this #{@selector_name()} query."


  # Returns the id for this query.
  id: ->
    unless @_id_value
      if @_entry_value
        @_id_value = @_entry_value[@id_name]
    @_id_value


  # Stores the given data.
  _store_query: (query) ->
    switch @selector_name()
      when 'entry'
        @_entry_value = query.entry
      when @entry_name
        @_entry_value = query[@entry_name]
      when 'id'
        @_id_value = query.id
      when @id_name
        @_id_value = query[@id_name]
      else
        throw new Error("Unknown selector name: #{@selector_name()}")


  # Returns the desired entry version described by this query.
  version: ->
    unless @_version_value
      if @_entry_value
        @_version_value = @_entry_value[@version_name] ? @_entry_value.version
    @_version_value

