#= require slender_data/cache
#= require slender_data/parameterizable

# Provides fast and convenient retrieval of cached objects
# using Queries.
#
# This class understands Queries and their specific configurations,
# and behaves accordingnly.
class slender_data.QueryCache extends slender_data.Parameterizable

  constructor: (params) ->
    super params

    # Storage for the entries.
    @cache = new slender_data.Cache


  # Adds the given entry to this cache.
  add: (entry) ->
    if @partial_entries
      return if @is_full_entry(@get entry: entry)
    @cache.add entry[@id_name], entry
    return

  # Adds a clone of the entry described by the given query.
  add_clone: (entry) ->
    clone = _.clone entry
    if @partial_entries
      existing_entry = @get entry: entry
      return existing_entry if @is_full_entry(existing_entry)
    @cache.add entry[@id_name], clone


  # Adds clones of the given entries to this cache.
  add_clones: (entries) ->
    @add_clone(entry) for entry in entries


  # Adds all the given entries to this cache.
  add_many: (entries) ->
    @add(entry) for entry in entries
    return


  # Adds clones of all entries described by the given multi-query.
  add_many_as_clones: (entries) ->
    @add_clone(entry) for entry in entries


  # Returns the entry corresponding to the given query.
  get: (query) ->
    query = @_querify_single query
    if (result = @cache.get query.id())
      if @partial_entries
        result = undefined if query.full and not @is_full_entry(result)
      if @versioning
        result = undefined if query.version() > new slender_data.SingleQuery(entry: result, @params).version()
    result

  # Returns all entries corresponding to the given multi-query.
  get_many: (multi_query) ->
    multi_query = @_querify_multi multi_query
    found = []
    missing = []
    for query in multi_query.transpose()
      if (entry = @get query)
        found.push entry
      else
        missing.push query.selector_value()
    found: found, missing: multi_query.replicate(missing)


  # Returns the number of stored entries.
  length: ->
    @cache.length()


  # Removes the entry described by the given query.
  remove: (query) ->
    query = @_querify_single query
    @cache.remove query.id()


  # Removes all entries described by the given multi-query.
  remove_many: (multi_query) ->
    multi_query = @_querify_multi multi_query
    @cache.remove_many multi_query.ids()

